/* Header */

#define CHANLEN		64
#define NICKLEN		22
#define USERLEN		10
#define	HOSTLEN		63

#define Time() time(NULL)

#define BUFSIZE 1024 /* DO NOT CHANGE THIS */

typedef struct Server_ Server;
typedef struct Protocol_ Protocol;
typedef struct Scan_ Scan;
typedef struct Stats_ Stats;
typedef struct Cache_ Cache;
typedef struct Bans_ Ban;

struct me {
	/* Local server info */
	char *name;
	char *bindip;
	char *descr;
	char *bladdr;
	char *logfile;

	FILE *logfd;

	int bindport;

	/* our socket listening on bindip:bindport */
	int bindfd;

	/* our running flags (command line params) */
	int runflags;

	/* our scan timeout in seconds */
	int timeout;

	/* Our start time */
	time_t started;

	/* Our Running pid */
	pid_t pid;

	/* Our local Admin info */
	char *admin1;
	char *admin2;
	char *admin3;

	/* our local die/restart passwords */
	char *diepass;
	char *respass;

	Ban *bans;
	Cache *negative;
} me;

struct Setting_ {
	char *banmsg;
	char *refer;
	char *scanmsg;

	int announce;
	int bantype;
	int tellclient;
} Setting;

struct Server_ {
	Server *next;

	char *name; /* the server name some.server.com */
	char *ip; /* the servers IP in full dotted octet 12.34.56.78 */
	char *pass; /* the password we use to link to the server */

	int port; /* the port we link on */
	int flags; /* the flags this server has enabled */
	int sockfd; /* FD */
	int state; /* Current state of the Server */
	time_t linktime; /* the time the server linked */
	int fails; /* we count how many Connect()'s failed, so we can disable the server and not hammer */

	Scan *scans;
	int scancount;

	Stats *stats;
};

/* Server flags */
#define FLAG_GOTPASS	0x001 /* we've recieved the PASS command through the socket */
#define FLAG_GOTSERVER	0x002 /* got the server command */
#define FLAG_DISABLE	0x004 /* When this is set, the server had failed negotiation, and link wont retry till a rehash */

/* Run Flags */
#define FLAGS_DEBUG	0x001
#define FLAGS_NODAEMON	0x002
#define FLAGS_STOP	0x004

/* Server State Flags */
#define SERV_CONNECTED		1 /* its connected and active */
#define SERV_DISCONNECTED	2 /* not connected */
#define SERV_HALFCONNECTED	3 /* Half Connected, we're connected, but no negotiation has gone way */

/* Scan state flags */
#define STATE_PENDING  	1 /* In queue to be scanned */
#define STATE_CONNECTED 2 /* Connected and waiting to be scanned */
#define STATE_SCANNING 	3 /* Currently Bieng Scanned */
#define STATE_DONE     	4 /* Done scanning, if this is set, the entry will disappear soon */

#define SCANTAB 100

struct Stats_ {
	int total;
	int positive;
	int negative;
	int blist;
	int timeouts;
	int incom;

	int clients;
};

struct Scan_ {
	Scan *next;

	char *ip; /* The IP in full dotted octet */
	char *irc_host; /* the IRC Hostname of the user being scanned */
	char *irc_nick; /* the IRC Nickname of the user being scanned */
	char *irc_user; /* the IRC Username of the user being scanned */

	time_t start; /* time first scan started */

	Protocol *protocols;
	int protcount;
	int done;

	int state;
};

struct Cache_ {
	char *ip;
	char *irc_host;
	char *type;

	int port;

	time_t added;
};

struct Protocol_ {
	Protocol *next;

	int port;
	int sockfd;
	int state;
	int (*func)();
	int announce;
	time_t conntime;

	struct sockaddr_in sockaddr;

	char *scantype;
};

struct Message {
	char *cmd;
	int (*func)();
};

struct Scans {
	int id;
	int port;
	char *type;
	int (*func)();
};

struct Bans_ {
	Ban *next;

	char *ip;
	char *mask;
	char *reason;
	char *realname;

	int timeout; /* if its temp, when will it be removed, if at all (0 = never) */

	time_t added; /* time it was added, if not static */
};

/* main.c */
extern int got_ping();
extern int got_pass();
extern int got_server();
extern int got_nick();
extern int got_version();
extern int got_info();
extern int got_stats();
extern int got_privmsg();
extern int got_uptime();

extern char *GetState(int);

/* misc.c */
extern void *Malloc(long);
extern char *strlower(char *);
extern void strip(char *);

/* socket.c */
extern void ReadLoop();
extern void readsock();
extern int OldConnect(char *, int, int);
extern int Connect(char *, char *, int, int);
extern int SendTo(int, char *, ...);
extern int SetupListen(char *, int, int);

/* server.c */
extern void add_serv_to_list();
extern void del_serv_from_list();
extern Server *AddServer();
extern int DelServer();
extern Server *findserver();

/* proxy.c */
extern int do_scans();
extern int scan_socks4();
extern int scan_socks5();
extern int scan_http();
extern int scan_wingate();
extern Protocol *AddProtocol();
extern void add_protocol_to_list();
extern Scan *AddScan();
extern int DelScan();
extern Scan *findscan();
extern void add_scan_to_list();
extern void del_scan_from_list();

#define VERSIONSTRING	"Soap 0.3-beta [dDfFl]"
