/* Header */

#include "includes.h"
#include "soap.h"
#include "config.h"

static char rcsid[] = "$Id: main.c,v 1.4 2002/09/20 03:31:42 crash Exp $";

Server *servers;
int servercount;
int FD_USE;

#define MSGTAB 100

struct Message msgtab[] = {
	{ "PING", got_ping },
	{ "PASS", got_pass },
	{ "SERVER", got_server },
	{ "NICK", got_nick },
	{ "VERSION", got_version },
	{ "INFO", got_info },
	{ "STATS", got_stats },
/*	{ "PRIVMSG", got_privmsg }, */
	{ "UPTIME", got_uptime },
	{ NULL, NULL }
};

char *configf = CONFIGNAME;
char *settingf = SETTINGNAME;
char *logfile = LOGFILE;

int main(int argc, char *argv[]) {
	int ss, flag;
	Server *s;
	pid_t tmppid;

	servercount = 0;
	me.bans = 0;
	FD_USE = 0;
	me.started = Time();
	if (me.bindfd > 0)
		close(me.bindfd);

	while(1) {
		flag = getopt(argc, argv, "Ddf:F:l:");
		if (flag == -1)
			break;
		
		switch(flag) {
			case 'D':
				log("DEBUG => Debug enabled");
				me.runflags |= FLAGS_DEBUG;
				break;
			case 'd':
				me.runflags |= FLAGS_NODAEMON;
				break;
			case 'f':
				configf = strdup(optarg);
				break;
			case 'F':
				settingf = strdup(optarg);
				break;
			case 'l':
				logfile = strdup(optarg);
				break;
			case '?':
			default:
				printf("Usage: %s [options]\n", argv[0]);
				printf("	Valid Options:\n");
				printf("	-D	Run in Debug mode. will print to debug.log unless used with -D\n");
				printf("	-d	Run in Nodaemon mode, soap will not fork() into the background\n");
				printf("	-f configfile	Use configfile rather then the default soap.conf\n");
				printf("	-F settingfile	Use settingfile rather then the default soap.settings\n");
				printf("	-l logfile	Use logfile rather then the default soap.log\n");
				exit(0);
		}
	}

/*	open_log(logfile); */

	if (read_conf(configf, 0) == -1)
		exit(-1);
	
	if (read_settings(settingf) == -1)
		exit(-1);

	setup_signals();

	if (!(me.runflags & FLAGS_NODAEMON)) {
		tmppid = fork();
		if (tmppid > 0) {
			me.pid = tmppid;
			log("INIT => Successfully forked into background with PID %d", me.pid);
			exit(0);
		}
		if (tmppid == -1) {
			log("FATAL => fork() Failed; %s", strerror(errno));
			exit(-1);
		}
	}
	else
		log("INIT => NODAEMON flag used, staying in foreground");

	while(!(me.runflags & FLAGS_STOP)) {
		ReadIrc();
		CheckServs();
		ReadScans();
	}

/*	fclose(me.logfd); */
}

int setup_signals() {
	signal(SIGPIPE, SIG_IGN);
}

int parse(char *buf, Server *serv) {
	char *parv[1024];
	int parc;
	int l = 0, i;

	if (me.runflags & FLAGS_DEBUG)
		log("RX => %s\n", buf);

	if (*buf == ':') {
		parc = str2arrayc(parv, buf+1, ' ');
		l = 1;
	}
	else
		parc = str2array(parv, buf, ' ');

	for(i=0; i<MSGTAB; i++) {
		if (!msgtab[i].cmd)
			break;

		if (strcasecmp(msgtab[i].cmd, parv[l]) == 0)
			(*msgtab[i].func)(parc, parv, serv);
	}
}

int send_connect(Server *serv) {
	SendTo(serv->sockfd, "PASS %s :TS", serv->pass);
	SendTo(serv->sockfd, "SERVER %s 1 :MusIRC Network Proxy Scanner", me.name);
	SendTo(serv->sockfd, "NICK SCAN 1 000005 +i SCAN %s %s :%s", me.name, me.name, VERSIONSTRING);
}

int got_ping(int parc, char *parv[], Server *serv) {
	SendTo(serv->sockfd, "PONG :%s", parv[1]);

}

int got_pass(int parc, char *parv[], Server *serv) {
	char *pass = parv[1];

	/* Accomodate for Servers that don't prefix pass with a : */
	if (*pass == ':')
		pass = pass+1;

	if (strcmp(pass, serv->pass) != 0) {
		log("ERROR => Connection closed to %s (invalid password)", serv->name);
		SendTo(serv->sockfd, "SQUIT %s :Invalid Pass", me.name);
		close(serv->sockfd);
		serv->state = SERV_DISCONNECTED;
	}
	else {
		serv->flags |= FLAG_GOTPASS;
		if (serv->flags & FLAG_GOTSERVER) {
			log("LINK => Link to %s Successful", serv->name);
			serv->linktime = Time();
			serv->state = SERV_CONNECTED;
		}
	}
}

int got_server(int parc, char *parv[], Server *serv) {
	/* Match our name against thiers */

	if (strcasecmp(parv[0], "SERVER"))
		return 0;

	if (strcasecmp(parv[1], serv->name) != 0) {
		log("ERROR => Connection closed to %s (name mismatch, Thiers: %s Ours: %s)", serv->name, parv[1], serv->name);
		SendTo(serv->sockfd, "SQUIT %s :Name Mismatch", me.name);
		close(serv->sockfd);
		serv->state = SERV_DISCONNECTED;
	}
	else {
		serv->flags |= FLAG_GOTSERVER;
		if (serv->flags & FLAG_GOTPASS) {
			log("LINK => Link to %s Successful", serv->name);
			serv->linktime = Time();
			serv->state = SERV_CONNECTED;
		}
	}
}

int got_nick(int parc, char *parv[], Server *serv) {
	char *nick;

	if (parc < 6) /* we should ignore these, but we need to change irc_nick in out Scan list */
		changenick(parv[0], parv[2], serv);

	else
		do_scans(parc, parv, serv);
}

int changenick(char *old, char *new, Server *serv) {
	Scan *sc;

	if (*old == ':')
		old = old+1;

	if (*new == ':')
		new = new+1;

	for (sc = serv->scans; sc; sc = sc->next) {
		if (!strcasecmp(old, sc->irc_nick)) {
			Free(sc->irc_nick);
			sc->irc_nick = strdup(new);
		}
	}
}

int got_version(int parc, char *parv[], Server *serv) {
	SendTo(serv->sockfd, ":%s 351 %s :%s", me.name, parv[0], VERSIONSTRING);
}

int got_info(int parc, char *parv[], Server *serv) {
	SendTo(serv->sockfd, ":%s 371 %s :%s by Crash (crash@urbox.org)", me.name, parv[0], VERSIONSTRING);
	SendTo(serv->sockfd, ":%s 371 %s :CVS: 'cvs -d :pserver:cvs@crash.urbox.org:/usr/home/cvs soap-devel'", me.name, parv[0]);
	SendTo(serv->sockfd, ":%s 371 %s :-------", me.name, parv[0]);
	SendTo(serv->sockfd, ":%s 371 %s :SOAP Scans incomming client across multiple IRC Servers for Anonymous Open proxies", me.name, parv[0]);
	SendTo(serv->sockfd, ":%s 371 %s :it effectivly scans Socks4/5, HTTP Connect, and Wingate proxies", me.name, parv[0]);
	SendTo(serv->sockfd, ":%s 371 %s :as well as checking for the host in a DNS Blacklist of known proxy addresses", me.name, parv[0]);
	SendTo(serv->sockfd, ":%s 371 %s :for additional Information visit our CVSWeb at http://cvs.urbox.org/cgi-bin/cvsweb.cgi/", me.name, parv[0]);
}

int got_stats(int parc, char *parv[], Server *serv) {
	Scan *scans;
	char c;
	int pp, bp, cs, np, tp, ip;

	c = *parv[2];


	switch(c) {
		case 'S':
			if (serv->scancount == 0)
				SendTo(serv->sockfd, ":%s NOTICE %s :No Current Scans for %s", me.name, parv[0], serv->name);
			for (scans = serv->scans; scans; scans = scans->next)
				SendTo(serv->sockfd, ":%s NOTICE %s :Scan for %s (%s!%s@%s) (%d/%d) {%d}", me.name, parv[0], scans->ip, scans->irc_nick, scans->irc_user, scans->irc_host, scans->done, scans->protcount, (Time() - scans->start));
			break;
		case 's':
			cs = serv->stats->clients / (Time() - serv->linktime);
			pp = (serv->stats->positive*100) / serv->stats->total;
			np = (serv->stats->negative*100) / serv->stats->total;
			bp = (serv->stats->blist*100) / serv->stats->total;
			tp = (serv->stats->timeouts*100) / serv->stats->total;
			serv->stats->incom = (serv->stats->total - (serv->stats->positive+serv->stats->negative+serv->stats->blist+serv->stats->timeouts));
			ip = (serv->stats->incom*100) / serv->stats->total;

			SendTo(serv->sockfd, ":%s NOTICE %s :I've scanned for %d proxies on %d hosts in %d seconds (%d connections/second)", me.name, parv[0], serv->stats->total, serv->stats->clients, (Time() - serv->linktime), cs);
			SendTo(serv->sockfd, ":%s NOTICE %s :%d were Positive Proxies (%d%%)", me.name, parv[0], serv->stats->positive, pp);
			SendTo(serv->sockfd, ":%s NOTICE %s :%d were negative Proxies (%d%%)", me.name, parv[0], serv->stats->negative, np);
			SendTo(serv->sockfd, ":%s NOTICE %s :%d were Listed in the Blacklist (%d%%)", me.name, parv[0], serv->stats->blist, bp);
			SendTo(serv->sockfd, ":%s NOTICE %s :%d had Timedout and were aborted (%d%%)", me.name, parv[0], serv->stats->timeouts, tp);
			SendTo(serv->sockfd, ":%s NOTICE %s : The Remaining %d are incomplete (%d%%)", me.name, parv[0], serv->stats->incom, ip);
			break;
		default:
			break;
	}
}

int got_privmsg(int parc, char *parv[], Server *serv) {
	Scan *sc;
	Protocol *pr;

/*
	if (!strcasecmp(parv[2], "SCAN")) {
		if (!(sc = findscan(parv[3], serv)))
			SendTo(serv->sockfd, ":%s PRIVMSG %s :No Scans for 
%s", "SCAN", parv[0], parv[3]);
		else
			for (pr = sc->protocols; pr; pr = pr->next)
				SendTo(serv->sockfd, ":%s PRIVMSG %s 
:Scanning for %s on %s:%d [%s] (%d)", "SCAN", parv[0], pr->scantype, 
sc->ip, pr->port, GetState(pr->state), (Time() - pr->conntime));
	}
*/

}

int got_uptime(int parc, char *parv[], Server *serv) {
	char utime[999];
	char ltime[999];

	get_dhm(utime, (Time() - me.started));
	get_dhm(ltime, (Time() - serv->linktime));

	SendTo(serv->sockfd, ":%s 330 %s %s :running for %s", me.name, parv[0], me.name, utime);
	SendTo(serv->sockfd, ":%s 330 %s %s :Linked for %s", me.name, parv[0], me.name, ltime);
}

char *GetState(int state) {
	switch(state) {
		case STATE_PENDING:
			return "Pending";
		case STATE_CONNECTED:
			return "Connected";
		case STATE_SCANNING:
			return "Scanning";
		case STATE_DONE:
			return "Done";
		default:
			return "Unknown";
	}
}
