/* Header */
#include "includes.h"
#include "soap.h"
#include "config.h"

static char rcsid[] = "$Id: proxy.c,v 1.5 2002/09/20 03:33:14 crash Exp $";

Server *servers;
int servercount;
int FD_USE;

struct pollfd ufds[MAXFDS+1];
int pollfds;

struct Scans scantab[] = {
        { 0, 1080, "Socks4", scan_socks4 },
        { 1, 1080, "Socks5", scan_socks5 },
        { 2, 3128, "HTTP", scan_http },
        { 3, 8080, "HTTP", scan_http },
	{ 4, 80, "HTTP", scan_http },
	{ 5, 23, "Wingate", scan_wingate },
        { -1, NULL, NULL, NULL }
};

int do_scans(int parc, char *parv[], Server *serv) {
	struct hostent *host;
	char *ip, *irc_addr, *irc_nick, *irc_user;
	int i;
	Scan *scan;

	if ((host = gethostbyname(parv[6])) == NULL) {
		log("NON-FATAL Unable to resolve host passed from server, aborting scan [%s]", parv[6]);
		return -1;
	}

	if ((ip = addr2ascii(host->h_addrtype, host->h_addr, host->h_length, NULL)) == NULL) {	
		log("NON-FATAL Unable to Make IP from hostent Structure [%s], aborting scan", parv[6]);
		return -1;
	}

	irc_addr = parv[6];
	irc_nick = parv[1];
	irc_user = parv[5];	

	if (Setting.tellclient == 1)
		SendTo(serv->sockfd, ":%s NOTICE %s :%s", me.name, irc_nick, Setting.scanmsg);

	if ((scan = AddScan(ip, serv)) == NULL)
		return -1;


	scan->irc_host = strdup(irc_addr);
	scan->irc_nick = strdup(irc_nick);
	scan->irc_user = strdup(irc_user);

	log("SCAN => Added %s (%s@%s) [%s], Adding Protocols {%s}", scan->irc_nick, scan->irc_user, scan->irc_host, ip, serv->name);

	serv->stats->clients++;

	if (CheckDNSBL(scan, serv) == 0) {
		AddProtocols(scan, serv);
	}
}

int CheckServs() {
	Server *s;

	for(s = servers; s; s = s->next) {
		switch(s->state) {
			case SERV_CONNECTED:
				s->fails = 0; /* Moved this here to fix a bug which hammered a server on some occasions */
				CheckScans(s);
				break;
			case SERV_HALFCONNECTED:
				break;

			case SERV_DISCONNECTED:
				if (!(s->flags & FLAG_DISABLE)) {
					if ((s->sockfd = Connect(me.bindip, s->ip, s->port, 1)) == -1) {
						log("SERVER => Connect() Failed to %s", s->name);
						s->fails++;
					}
					else if (s->fails > MAXFAILS)
						s->flags = FLAG_DISABLE;
					else {
						log("LINKING => Linking to %s [%s:%d]", s->name, s->ip, s->port);
						send_connect(s);
						s->state = SERV_HALFCONNECTED;
					}
				}
				break;
			default:
				break;
		}
	}
}

int SetDone(Protocol *pr, Scan *sc) {
	if (pr->state != STATE_DONE) {
		pr->state = STATE_DONE;
		sc->done++;
		CloseProt(pr);
	}
}

int CheckScans(Server *serv) {
	Scan *sc;
	Protocol *pr;

	pollfds = 0;
	for (sc = serv->scans; sc; sc = sc->next) {
		for (pr = sc->protocols; pr; pr = pr->next) {
			if (pr->state != STATE_PENDING || pr->state != STATE_DONE) {
				ufds[pollfds].events = 0;
				ufds[pollfds].revents = 0;
				ufds[pollfds].fd = pr->sockfd;
			
				ufds[pollfds].events |= POLLHUP;
				ufds[pollfds].events |= POLLNVAL;
			}

			switch(pr->state) {
				case STATE_PENDING:
					if (sc->start == 0)
						sc->start = Time();

					if (FD_USE < MAXFDS)
						ScanConnect(sc, pr);
							
					break;
				case STATE_CONNECTED:
					if ((Time() - pr->conntime) >= me.timeout) {
						SetDone(pr, sc);
						serv->stats->timeouts++;
					}
					else {
						ufds[pollfds].events |= POLLOUT;
					}
						
					break;
				case STATE_SCANNING:
					if ((Time() - pr->conntime) >= me.timeout) {
						SetDone(pr, sc);
						serv->stats->timeouts++;
					}
					else {
						ufds[pollfds].events |=  POLLIN;
					}
					break;
				case STATE_DONE:
					if (sc->done >= sc->protcount && !pr->next) {
						log("COMPLETE => Scan of %s Completed in %d Seconds (%d/%d)", sc->ip, (Time() - sc->start), sc->done, sc->protcount);
						if (DelScan(sc->ip, serv) == -1)
							log("ERROR => DelScan() Failed for %s", sc->ip);
					}

					break;
				default:
					break;
			}
		if (++pollfds > MAXFDS)
			break;
		}
	}
}

int scan_socks4(Scan *scan, Protocol *protocol, Server *serv) {
	struct in_addr addr;
	unsigned long laddr;
	int len;

	if (inet_aton(me.bindip, &addr) == 0) {
		log("INET => %s is not a Valid IP", me.bindip);
		return 0;
	}

	laddr = htonl(addr.s_addr);

	len = SendScan(protocol->sockfd, "%c%c%c%c%c%c%c%c%c", 4, 1, (((unsigned short)me.bindport) >> 8) & 0xFF,
		 (((unsigned short)me.bindport) & 0xff), (char) (laddr >> 24) & 0xFF, (char) (laddr >> 16) & 0xFF,
		 (char) (laddr >> 8) & 0xFF, (char) laddr & 0xFF, 0);

	if (len <= 0) {
		SetDone(protocol, scan);
		serv->stats->negative++;
	}

	else
		protocol->state = STATE_SCANNING;
}

int scan_socks5(Scan *scan, Protocol *protocol, Server *serv) {
	struct in_addr addr;
	unsigned long laddr;
	int len;

	if (inet_aton(me.bindip, &addr) == 0) {
		log("INET => %s is not a Valid IP", me.bindip);
		return 0;
	}

	laddr = htonl(addr.s_addr);

	len = SendScan(protocol->sockfd, "%c%c%c", 5, 1, 0); /* Send Version 5, 1 method, method 0 (no auth) */
	if (len <= 0) {
		SetDone(protocol, scan);
		serv->stats->negative++;
		return 0;
	}

	len = SendScan(protocol->sockfd, "%c%c%c%c%c%c%c%c%c%c", 5, 1, 0, 1, (char)(laddr >> 24) & 0xFF, (char)(laddr >> 16) & 0xFF,
			(char)(laddr >> 8) & 0xFF, (char)laddr & 0xFF, (((unsigned short)me.bindport) >> 8) & 0xFF,
			(((unsigned short)me.bindport) &0xFF));
	if (len <= 0) {
		SetDone(protocol, scan);
		serv->stats->negative++;
		return 0;
	}

	protocol->state = STATE_SCANNING;
}

int scan_wingate(Scan *scan, Protocol *protocol, Server *serv) {
	if (SendScan(protocol->sockfd, "%s:%d\n\r", me.bindip, me.bindport) <= 0) {
		SetDone(protocol, scan);
		serv->stats->negative++;
	}
	else {
		log("WINGATE => Sent scan sequence to %s:%d", scan->ip, protocol->port);
		protocol->state = STATE_SCANNING;
	}
}

int scan_http(Scan *scan, Protocol *protocol, Server *serv) {
	if (SendScan(protocol->sockfd, "CONNECT %s:%d HTTP/1.0\r\n\r\n", me.bindip, me.bindport) <= 0) {
		/* Write Failed, no proxy */
		SetDone(protocol, scan);
		serv->stats->negative++;
	}
	else {
		log("HTTP => Sent scan connect to %s:%d", scan->ip, protocol->port);
		protocol->state = STATE_SCANNING;
	}
}

/* To Assist in keeping FD_USE correct */
int CloseProt(Protocol *pr) {
	close(pr->sockfd);
	FD_USE--;
}

int kline_proxy(Server *serv, Scan *scan, int dnsbl) {
	Protocol *pr; /* Take a wild guess at the protocol, this will be done with accuracy in the future */

	if (dnsbl == 1) {
		log("FOUND => %s in DNSBL %s", scan->ip, me.bladdr);
		if (Setting.announce == 1)
		SendTo(serv->sockfd, ":%s OPERWALL :Proxy DNSBL entry found for %s (%s!%s@%s)", me.name, scan->ip, scan->irc_nick, scan->irc_user, scan->irc_host);
		SendTo(serv->sockfd,":SCAN KLINE * 86400 * %s :%s",
		scan->ip, Setting.banmsg);
		DelScan(scan->ip, serv);
		serv->stats->blist++;
		return 0;
	}
	else
		serv->stats->positive++;

	for (pr = scan->protocols; pr; pr = pr->next) {
		if (pr->state == STATE_SCANNING)
			break;
	}

	log("FOUND => %s Proxy found (%s!%s@%s) [%s:%d]", pr->scantype, scan->irc_nick, scan->irc_user, scan->irc_host, scan->ip, pr->port);

	if (Setting.announce == 1)
		SendTo(serv->sockfd, ":%s OPERWALL :%s Proxy Found (%s!%s@%s) [%s:%d]", me.name, pr->scantype, scan->irc_nick, scan->irc_user, scan->irc_host, scan->ip, pr->port);
                SendTo(serv->sockfd,":SCAN KLINE * 86400 * %s :%s",
                scan->ip, Setting.banmsg);

	DelScan(scan->ip, serv);
}

/* Clean ALL scan links from a server, this is called when a server link is lost */
int clean_scans(Server *serv) {
	Scan *sc;

	for (sc = serv->scans; sc; sc = sc->next)
		DelScan(sc->ip, serv);
}

Scan *AddScan(char *ip, Server *serv) {
	Scan *new;
	int i;

	if (!ip) {
		log("NON-FATAL => NULL IP Passed to AddScan(), aborting");
		return NULL;
	}

	if (findscan(ip, serv) != NULL) {
		log("INFO => %s Already exists in Scan table", ip);
		return NULL;
	}

	new = (Scan *)Malloc(sizeof(*new));

	new->ip = strdup(ip);

	new->protocols = 0;
	new->protcount = 0;
	new->done = 0;
	new->start = 0;

	add_scan_to_list(new, serv);

	serv->scancount++;

	return new;
}

int AddProtocols(Scan *sc, Server *serv) {
	int i;
	Protocol *cur;

	for(i=0; i<SCANTAB; i++) {
		if (scantab[i].id == -1)
			break;

                if ((cur = AddProtocol(sc, serv)) == NULL) {
                        log("PROT => Failed to Add protocol to %s", sc->ip);
                        break;
                }

		cur->scantype = strdup(scantab[i].type);

		/* Setup sockaddr */
		memset(&(cur->sockaddr), 0, sizeof(struct sockaddr_in));

		cur->sockaddr.sin_family = AF_INET;
		cur->sockaddr.sin_port = htons(scantab[i].port);
		cur->sockaddr.sin_addr.s_addr = inet_addr(sc->ip);

		cur->port = scantab[i].port;
		cur->func = &(*scantab[i].func);
		serv->stats->total++;
	}

	return 1;
}

Protocol *AddProtocol(Scan *sc, Server *serv) {
	Protocol *new;

	new = (Protocol *)Malloc(sizeof(*new));

	new->state = STATE_PENDING;

	add_protocol_to_list(new, sc);

	return new;
}

void add_protocol_to_list(Protocol *pro, Scan *sc) {
	Protocol *pr;

	if (!sc->protocols) {
		pro->next = 0;
		sc->protocols = pro;
	}

	else {
		for (pr = sc->protocols; pr; pr = pr->next) {
			if (!pr->next) {
				pro->next = 0;
				pr->next = pro;
				break;
			}
		}
	}
	sc->protcount++;

}

void add_scan_to_list(Scan *newscan, Server *serv) {
	Scan *sc;

	if (!serv->scans) {
		newscan->next = 0;
		serv->scans = newscan;
	}
	else {
		for (sc = serv->scans; sc; sc = sc->next) {
			if (!sc->next) {
				newscan->next = 0;
				sc->next = newscan;
				break;
			}
		}
	}
	serv->scancount++;
}

int DelScan(char *ip, Server *serv) {
	Scan *tmp;

	if (!ip)
		return -1;

	if ((tmp = findscan(ip, serv)) == NULL)
		return -1;

	del_scan_from_list(tmp, serv);

	serv->scancount--;

	return 0;
}

void del_scan_from_list(Scan *del, Server *serv) {
	Scan *sc;
	Scan *last;
	Protocol *p;

	last = 0;
	
	for (p = del->protocols; p; p = p->next) {
		Free(p->scantype);
		Free(p);
	}

	for (sc = serv->scans; sc; sc = sc->next) {
		if (sc == del) {
			if (last == 0)
				serv->scans = sc->next;
			else
				last->next = sc->next;

			Free(sc->irc_nick);
			Free(sc->irc_user);
			Free(sc->irc_host);
			Free(sc->ip);
			Free(sc);

			break;
		}
		last = sc;
	}
	serv->scancount--;
}

Scan *findscan(char *ip, Server *serv) {
        Scan *sc;

	if (!serv)
		return NULL;

	for (sc = serv->scans; sc; sc = sc->next)
		if (!strcmp(sc->ip, ip))
			return sc;

	return NULL;
}
