/* Header */

#include "includes.h"
#include "config.h"
#include "soap.h"
#include <fcntl.h>
#include <errno.h>

static char rcsid[] = "$Id: socket.c,v 1.3 2002/09/20 03:07:24 crash Exp $";

Server *servers;
int servercount;

struct pollfd ufds[MAXFDS+1];
int pollfds;

int FD_USE;

int Connect(char *vhostname, char *hostname, int port, int block) {
        int s;
        struct hostent *hv, *hn;
        struct sockaddr_in sa;  
        
        if ((hn = gethostbyname (hostname)) == NULL) {
		log("NON-FATAL => gethostbyname() failed for %s; %s", hostname, strerror(errno));
                return (-1);
	}
                
        if ((hv = gethostbyname (vhostname)) == NULL) {
		log("NON-FATAL => gethostbyname() failed for %s; %s", hostname, strerror(errno));
                return (-1);
	}
                
        if ((s = socket(AF_INET, SOCK_STREAM, 0)) < 0) {
		log("NON-FATAL => socket() failed for %s; %s", strerror(errno));
                return (-1);
	}
                
        bzero(&sa, sizeof(sa));
        sa.sin_family = AF_INET;
	sa.sin_port = 0;
        memcpy (&sa.sin_addr, hv->h_addr, hv->h_length);
        sa.sin_addr.s_addr = inet_addr(vhostname);
        
        if (bind(s, (struct sockaddr *) &sa, sizeof(struct sockaddr_in)) < 0) {
		log("NON-FATAL => bind() Failed for %s:%d; %s", vhostname, ntohs(sa.sin_port), strerror(errno));
                return (-1);
        }
         
        bzero(&sa, sizeof(sa));
        sa.sin_family = AF_INET;
        sa.sin_port = htons(port);
        memcpy (&sa.sin_addr, hn->h_addr, hn->h_length);
        sa.sin_addr.s_addr = inet_addr(hostname);
        
	if (block == 0)
		fcntl(s, F_SETFL, O_NONBLOCK);

        if (connect (s, (struct sockaddr *) &sa, sizeof(sa)) < 0) {
                close(s);
		log("NON-FATAL => Connect() Failed to %s:%d; %s", hostname, port, strerror(errno));
                return (-1);
        }


        return s;
}

int ScanConnect(Scan *scan, Protocol *prot) {
	struct sockaddr_in bindaddr;

	memset(&bindaddr, 0, sizeof(struct sockaddr_in));

	if (!inet_aton(me.bindip, &(bindaddr.sin_addr))) {
		log("FATAL => Invalid IP %s in M:Line", me.bindip);
		exit(-1);
	}

	bindaddr.sin_family = AF_INET;
	bindaddr.sin_port = 0;

	prot->sockfd = socket(PF_INET, SOCK_STREAM, 0);

	FD_USE++;

	if (prot->sockfd == -1) {
		log("SCAN => Error Allocating sockfd for %s:%d; %s", scan->ip, prot->port, strerror(errno));
		SetDone(prot, scan);
		return;
	}

	if (bind(prot->sockfd, (struct sockaddr *)&bindaddr, sizeof(struct sockaddr_in)) == -1) {
		log("SCAN => bind() Failed; %s", strerror(errno));
		exit(-1);
	}

	prot->conntime = Time();

	fcntl(prot->sockfd, F_SETFL, O_NONBLOCK);

	connect(prot->sockfd, (struct sockaddr *)&(prot->sockaddr), sizeof(prot->sockaddr));

	prot->state = STATE_CONNECTED;

	return 0;	
}

int SetupListen(char *ip, int port, int block) {
	int sockfd;
	struct hostent *h;
	struct sockaddr_in addr;

	if ((sockfd = socket(AF_INET, SOCK_STREAM, 0)) == -1) {
		log("FATAL => socket() Failed for Listen Socket on %s:%d; %s", ip, port, strerror(errno));
		exit(-1);
	}

	if ((h = gethostbyname(ip)) == NULL) {
		log("FATAL => Resolution for %s failed for Listen Socket; %s", ip, strerror(errno));
		exit(-1);
	}

	addr.sin_family = AF_INET;
	addr.sin_port = htons(port);
	addr.sin_addr = *((struct in_addr *)h->h_addr);
	memset(&(addr.sin_zero), '\0', 8);

	if (bind(sockfd, (struct sockaddr *)&addr, sizeof(struct sockaddr)) == -1) {
		log("FATAL => bind() Failed for Listen Socket on %s:%d; %s", ip, port, strerror(errno));
		exit(-1);
	}

	if (block == 0)
		fcntl(sockfd, F_SETFL, O_NONBLOCK);

	if (listen(sockfd, LISTEN_BACKLOG) == -1) {
		log("FATAL => listen() failed for Listen Socket on %s:%d; %s", ip, port, strerror(errno));
		exit(-1);
	}

	return sockfd;
}

void ReadIrc() {
	Server *s;
	struct timeval irctv;
	fd_set ircread;
	int maxfd = 0;

	FD_ZERO(&ircread);
	irctv.tv_sec = 0;
	irctv.tv_usec = 50000;

	if (me.bindfd > 0) {
		FD_SET(me.bindfd, &ircread);
		if (me.bindfd > maxfd)
			maxfd = me.bindfd;
	}

	for (s = servers; s; s = s->next) 
		if (s->state == SERV_CONNECTED || s->state == SERV_HALFCONNECTED) {
			FD_SET(s->sockfd, &ircread);
			if (maxfd < s->sockfd)
				maxfd = s->sockfd;
		}

	if ((select(maxfd+1, &ircread, 0, 0, &irctv)) > 0) {
		for (s = servers; s; s = s->next)
			if ((s->state == SERV_CONNECTED || s->state == SERV_HALFCONNECTED) && FD_ISSET(s->sockfd, &ircread))	
				readsock(s);
	}

	if (me.bindfd > 0 && FD_ISSET(me.bindfd, &ircread))
		Accept(me.bindfd);
}

void ReadScans()
{
        int i = 0, t; 
	Server *s;
	Scan *sc;
	Protocol *pr;

	if ((t = poll(ufds, pollfds, 0)) > 0) {
		for (s = servers; s; s = s->next) {
			for (sc = s->scans; sc; sc = sc->next) {
				for (pr = sc->protocols; pr; pr = pr->next) {
					for(i = 0; i < pollfds; i++) {
						if (ufds[i].fd == pr->sockfd) {
							if (ufds[i].revents & POLLOUT) {
								(*pr->func)(sc, pr, s);
							}

							else if (ufds[i].revents & POLLIN) {
								ReadScan(pr, sc, s);
							}

							else if (ufds[i].revents & POLLHUP) {
								SetDone(pr, sc);
							}
						}
					}
				}
			}
               	}
	}
	else {
		switch (t) {
			case EFAULT:
				log("POLL => FDS Point outside allocation space");
				break;
			case EINTR:
				log("POLL => Timeout");
				break;
			case EINVAL:
				log("POLL => The specified time limit is negative");
				break;
			case 0:
				break;
			default:
				log("POLL => Failed; %d", t);
				break;
		}
	}
}

int ReadScan(Protocol *prot, Scan *sc, Server *serv) {
	char buf[1024];

	switch(recv(prot->sockfd, buf+1, 1, 0)) {
		case 0:
			SetDone(prot, sc);
			break;
		case -1:
			log("READ => Read on %s:%d failed; %s", sc->ip, prot->port, strerror(errno));
			break;
		default:
			return;
		
	}
}

int Accept(int sock) {
	int accfd;
	struct sockaddr_in thiers;
	int addr_size;
	char *ip;
	Scan *scan;
	Protocol *pr;
	Server *serv;

	if ((accfd = accept(sock, (struct sockaddr *)&thiers, &addr_size)) == -1) {
		log("NON-FATAL => Accept() Failed on accept()");
		return -1;
	}

	ip = strdup(inet_ntoa(thiers.sin_addr));

	log("INFO => Got connection on %s:%d from %s:%d", me.bindip, me.bindport, ip, ntohs(thiers.sin_port));

	for(serv = servers; serv; serv = serv->next)
		for (scan = serv->scans; scan; scan = scan->next)
			if (!strcmp(ip, scan->ip)) { /* Positive Proxy, ban it */
				kline_proxy(serv, scan, 0);
				close(accfd);
				Free(ip);
				return 0;
			}

	/* Not a proxy, close connection */

	close(accfd);

	Free(ip);
}
         
void readsock(Server *serv) {
        char buf[1024];  
        int i = 0, sock = serv->sockfd, bytes;

        while ((bytes = recv(sock, buf+i, 1, 0)) == 1) {
		if (buf[i] == '\n' || i >= 1024)
			break;
		i++;
        }
	if (bytes != 1) {
		/* Errno on the Socket */
		if (bytes == 0) {
			/* Remote side closed connection */
			log("NON-FATAL => Server %s Closed the Connection", serv->name);
			serv->state = SERV_DISCONNECTED;
			clean_scans(serv);
			close(serv->sockfd);
			serv->fails++;
		}
		else
			log("NON-FATAL => Recv() Failed; %s", strerror(errno));
	}
		

        strip(buf);
        
	parse(buf, serv);
}

int SendScan(int sock, char *fmt, ...) {
	va_list ap;
	char buf[BUFSIZE];
	int len;

	va_start(ap, fmt);
	vsnprintf(buf, BUFSIZE, fmt, ap);
	va_end(ap);

	len = send(sock, buf, strlen(buf), 0);

	return len;
}

int SendTo(int sock, char *fmt,...)
{
        va_list ap;
        char buf[BUFSIZE];
	int len;

        va_start(ap, fmt);
        vsnprintf(buf, BUFSIZE, fmt, ap);
	va_end(ap);

	if (me.runflags & FLAGS_DEBUG)
		printf("TX: %s\n", buf);

        strcat(buf, "\n");

        if ((len = send(sock, buf, strlen(buf), 0)) == -1)
		log("SEND => Error Writting; %s", strerror(errno));

	return len;
}
